# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Classes

* [`ipset`](#ipset): module to install the ipset tooling and to manage individual ipsets

### Defined types

* [`ipset::set`](#ipset--set): Declare an IP Set.
* [`ipset::unmanaged`](#ipset--unmanaged): Declare an IP set, without managing its content.  Useful when you have a dynamic process that generates an IP set content, but still want to 

### Data types

* [`IPSet::Options`](#IPSet--Options): list of options you can configure on an ipset
* [`IPSet::Set::Array`](#IPSet--Set--Array): type to allow an array of ip addresses
* [`IPSet::Set::File_URL`](#IPSet--Set--File_URL): type to allow a static file on the target system as source for ipsets
* [`IPSet::Set::Puppet_URL`](#IPSet--Set--Puppet_URL): type to allow a file on the puppetserver as source for ip addresses for ipsets
* [`IPSet::Settype`](#IPSet--Settype): different datatypes that provides prefixes for the actual ipset
* [`IPSet::Type`](#IPSet--Type): type to allow all different hash setups for ipsets

## Classes

### <a name="ipset"></a>`ipset`

module to install the ipset tooling and to manage individual ipsets

#### Parameters

The following parameters are available in the `ipset` class:

* [`packages`](#-ipset--packages)
* [`service`](#-ipset--service)
* [`service_ensure`](#-ipset--service_ensure)
* [`enable`](#-ipset--enable)
* [`firewall_service`](#-ipset--firewall_service)
* [`package_ensure`](#-ipset--package_ensure)
* [`config_path`](#-ipset--config_path)
* [`sets`](#-ipset--sets)

##### <a name="-ipset--packages"></a>`packages`

Data type: `Array[String[1]]`

The name of the package we want to install

##### <a name="-ipset--service"></a>`service`

Data type: `String[1]`

The name of the service that we're going to manage

##### <a name="-ipset--service_ensure"></a>`service_ensure`

Data type: `Boolean`

Desired state of the service. If true, the service will be running. If false, the service will be stopped

##### <a name="-ipset--enable"></a>`enable`

Data type: `Boolean`

Boolean to decide if we want to have the service in autostart or not

##### <a name="-ipset--firewall_service"></a>`firewall_service`

Data type: `Optional[Pattern[/\.service$/]]`

An optional service name. if provided, the ipsets will be configured before this. So your firewall will depend on the chains. The name should end with `.service`. This is only supported on systemd-based Operating Systems

Default value: `undef`

##### <a name="-ipset--package_ensure"></a>`package_ensure`

Data type: `Enum['present', 'absent', 'latest']`

ensure parameter for the ipset package resource

##### <a name="-ipset--config_path"></a>`config_path`

Data type: `Stdlib::Absolutepath`

path to the directory for the ipsets

##### <a name="-ipset--sets"></a>`sets`

Data type: `Hash`

Hash Hash of 'ipset::set' resources

Default value: `{}`

## Defined types

### <a name="ipset--set"></a>`ipset::set`

Declare an IP Set.

#### Examples

##### An IP set containing individual IP addresses, specified in the code.

```puppet
ipset::set { 'a-few-ip-addresses':
  set => ['10.0.0.1', '10.0.0.2', '10.0.0.42'],
}
```

##### An IP set containing IP networks, specified with Hiera.

```puppet
ipset::set { 'hiera-networks':
  set  => lookup('foo', IP::Address::V4::CIDR),
  type => 'hash:net',
}
```

##### An IP set of IP addresses, based on a file stored in a module.

```puppet
ipset::set { 'from-puppet-module':
  set => "puppet:///modules/${module_name}/ip-addresses",
}
```

##### An IP set of IP networks, based on a file stored on the filesystem.

```puppet
ipset::set { 'from-filesystem':
  set => 'file:///path/to/ip-addresses',
}
```

##### setup multiple ipsets based on a hiera hash with multiple arrays and multiple IPv4/IPv6 prefixes. Use the voxpupuli/ferm module to create suitable iptables rules.

```puppet
$ip_ranges = lookup('ip_net_vlans').flatten.unique
$ip_ranges_ipv4 = $ip_ranges.filter |$ip_range| { $ip_range =~ Stdlib::IP::Address::V4 }
$ip_ranges_ipv6 = $ip_ranges.filter |$ip_range| { $ip_range =~ Stdlib::IP::Address::V6 }

ipset::set{'v4':
  ensure => 'present',
  set    => $ip_ranges_ipv4,
  type   => 'hash:net',
}

ipset::set{'v6':
  ensure  => 'present',
  set     => $ip_ranges_ipv6,
  type    => 'hash:net',
  options => {
    'family' => 'inet6',
  },
}

ferm::ipset{'INPUT':
  ip_version => 'ip6',
  sets       => {
    'v6' => 'ACCEPT',
  },
}

ferm::ipset{'INPUT':
  ip_version => 'ip',
  sets       => {
    'v4' => 'ACCEPT',
  },
}
```

#### Parameters

The following parameters are available in the `ipset::set` defined type:

* [`set`](#-ipset--set--set)
* [`ensure`](#-ipset--set--ensure)
* [`type`](#-ipset--set--type)
* [`options`](#-ipset--set--options)
* [`ignore_contents`](#-ipset--set--ignore_contents)
* [`keep_in_sync`](#-ipset--set--keep_in_sync)

##### <a name="-ipset--set--set"></a>`set`

Data type: `IPSet::Settype`

IP set content or source.

##### <a name="-ipset--set--ensure"></a>`ensure`

Data type: `Enum['present', 'absent']`

Should the IP set be created or removed ?

Default value: `'present'`

##### <a name="-ipset--set--type"></a>`type`

Data type: `IPSet::Type`

Type of IP set.

Default value: `'hash:ip'`

##### <a name="-ipset--set--options"></a>`options`

Data type: `IPSet::Options`

IP set options.

Default value: `{}`

##### <a name="-ipset--set--ignore_contents"></a>`ignore_contents`

Data type: `Boolean`

If ``true``, only the IP set declaration will be
managed, but not its content.

Default value: `false`

##### <a name="-ipset--set--keep_in_sync"></a>`keep_in_sync`

Data type: `Boolean`

If ``true``, Puppet will update the IP set in the kernel
memory. If ``false``, it will only update the IP sets on the filesystem.

Default value: `true`

### <a name="ipset--unmanaged"></a>`ipset::unmanaged`

Declare an IP set, without managing its content.

Useful when you have a dynamic process that generates an IP set content,
but still want to define and use it from Puppet.

<aside class="warning">
When changing IP set attributes (type, options) contents won't be kept,
set will be recreated as empty.
</aside>

#### Examples

##### 

```puppet
ipset::unmanaged { 'unmanaged-ipset-name': }
```

#### Parameters

The following parameters are available in the `ipset::unmanaged` defined type:

* [`ensure`](#-ipset--unmanaged--ensure)
* [`type`](#-ipset--unmanaged--type)
* [`options`](#-ipset--unmanaged--options)
* [`keep_in_sync`](#-ipset--unmanaged--keep_in_sync)

##### <a name="-ipset--unmanaged--ensure"></a>`ensure`

Data type: `Enum['present', 'absent']`

Should the IP set be created or removed ?

Default value: `'present'`

##### <a name="-ipset--unmanaged--type"></a>`type`

Data type: `IPSet::Type`

Type of IP set.

Default value: `'hash:ip'`

##### <a name="-ipset--unmanaged--options"></a>`options`

Data type: `IPSet::Options`

IP set options.

Default value: `{}`

##### <a name="-ipset--unmanaged--keep_in_sync"></a>`keep_in_sync`

Data type: `Boolean`

If ``true``, Puppet will update the IP set in the kernel
memory. If ``false``, it will only update the IP sets on the filesystem.

Default value: `true`

## Data types

### <a name="IPSet--Options"></a>`IPSet::Options`

list of options you can configure on an ipset

* **See also**
  * http://ipset.netfilter.org/ipset.man.html#lbAI

Alias of

```puppet
Struct[{
    Optional[family]   => Enum['inet', 'inet6'],
    Optional[hashsize] => Integer[128],
    Optional[maxelem]  => Integer[128],
    Optional[netmask]  => Stdlib::IP::Address,
    Optional[timeout]  => Integer[1],
}]
```

### <a name="IPSet--Set--Array"></a>`IPSet::Set::Array`

type to allow an array of ip addresses

Alias of `Array[String]`

### <a name="IPSet--Set--File_URL"></a>`IPSet::Set::File_URL`

type to allow a static file on the target system as source for ipsets

Alias of `Pattern[/^file:\/\/\//]`

### <a name="IPSet--Set--Puppet_URL"></a>`IPSet::Set::Puppet_URL`

type to allow a file on the puppetserver as source for ip addresses for ipsets

Alias of `Pattern[/^puppet:\/\//]`

### <a name="IPSet--Settype"></a>`IPSet::Settype`

different datatypes that provides prefixes for the actual ipset

Alias of `Variant[IPSet::Set::Array, IPSet::Set::Puppet_URL, IPSet::Set::File_URL, String]`

### <a name="IPSet--Type"></a>`IPSet::Type`

type to allow all different hash setups for ipsets

* **See also**
  * http://ipset.netfilter.org/ipset.man.html#lbAW
  * documentation for all different hash options

Alias of `Enum['hash:ip', 'hash:ip,port', 'hash:ip,port,ip', 'hash:ip,port,net', 'hash:ip,mark', 'hash:net', 'hash:net,net', 'hash:net,iface', 'hash:net,port', 'hash:net,port,net', 'hash:mac']`

